// ACWatch - monitors a list of files for changes in last access time, 
// and takes action when a change is noticed
// Copyright (c) 2003 by Tom Grandgent

#include "acwatch.h"		// Defines, types
#include "acwGlobals.h"		// Global variables

// Logging function
void Log(char *text, ...)
{
	va_list ap;
	char s[4096], s2[4096];
	static int startTime = 0;
	DWORD curtime;

	if (!startTime)
	{
		timeBeginPeriod(1);
		startTime = timeGetTime();
		timeEndPeriod(1);
	}

	{
		// Parse the variable arguments and come up with one string
		va_start(ap, text);
		vsprintf(s, text, ap);
		va_end(ap);

		if (s[0] && s[strlen(s) - 1] == '\n')
			s[strlen(s) - 1] = '\0';

		timeBeginPeriod(1);
		curtime = timeGetTime();
		timeEndPeriod(1);
		sprintf(s2, "[%d] ", curtime - startTime);
		strcat(s2, s);

		// If the main window is open, add this log line to the list box
		if (hwndLBLog && !bMinimized)
		{
			// Add the log line to the list box
			SendMessage(hwndLBLog, LB_ADDSTRING, 0, (LPARAM) s2);
			// Select the last item in the list box to scroll to the bottom
			int itemCount = SendMessage(hwndLBLog, LB_GETCOUNT, 0, 0);
			SendMessage(hwndLBLog, LB_SETCURSEL, (WPARAM) itemCount - 1, 0);
			// Remove the selection
			SendMessage(hwndLBLog, LB_SETCURSEL, (WPARAM) -1, 0);
		}

		//printf("%s\n", s2);

#ifdef _DEBUG
	OutputDebugString(s2);
	OutputDebugString("\n");
#endif

		FILE *fp = fopen(logFilename, "a");
		if (fp)
		{
			fprintf(fp, "[%d] %s\n", curtime - startTime, s);
			fclose(fp);
		}
	}
}

// Gets the base filename.. for example, given "c:\blah.txt", it returns "blah.txt"
char *getBaseFilename(char *fn)
{
	char *ch = strrchr(fn, '\\');
	if (ch)
		return ch + 1;
	else
		return fn;
}

// Returns a temporary string which is the combination of the two given strings
char *combinedString(char *str1, char *str2)
{
	static char *statstr = NULL;
	static int allocedLen = -1;
	
	if (!str1)
		return str2;
	if (!str2)
		return str1;

	int len1 = strlen(str1);
	int len2 = strlen(str2);

	if (len1 + len2 + 2 > allocedLen)
	{
		allocedLen = len1 + len2 + 2;
		statstr = (char *) realloc(statstr, allocedLen);
	}

	strcpy(statstr, str1);
	strcat(statstr, str2);

	return statstr;
}

// Initialize Winsock
int init_winsock(void)
{
	WORD wVersionRequested;		// Used for Winsock initialization
	WSADATA wskData;			// Used for Winsock initialization
	int err;

	// Initialize Winsock
	wVersionRequested = MAKEWORD(1, 1);
	err = WSAStartup(wVersionRequested, &wskData);
	if (err)
	{
		printf("ERR: Winsock failed to initialized - WSAStartup() returned %d", err);
		return 1;
	}

	return 0;
}

// Shutdown Winsock
void shutdown_winsock(void)
{
	WSACleanup();
}

// Adds the files in a directory, and all of its subdirectories recursively...
void monitorDirectory(char *dirname, int depth)
{
	char searchpath[MAX_PATH], fullpath[MAX_PATH];
	struct _finddata_t fd;
	long hFile;

	//Log("Adding directory (%d): %s", depth, dirname);

	if (!dirname)
		return;

	// Set up the search path
	strcpy(searchpath, dirname);
	if (dirname[strlen(dirname) - 1] != '\\')
		strcat(searchpath, "\\");
	strcat(searchpath, "*");

	// Enumerate all files and directories
	hFile = _findfirst(searchpath, &fd);
	while (1)
	{
		// Recurse on subdirectories
		if (fd.attrib & _A_SUBDIR)
		{
			// Ignore "." and ".." entries
			if (stricmp(fd.name, ".") && stricmp(fd.name, ".."))
			{
				// Build complete path to recurse on
				strcpy(fullpath, dirname);
				strcat(fullpath, "\\");
				strcat(fullpath, fd.name);
				// Recurse on this subdirectory
				monitorDirectory(fullpath, depth + 1);
			}
		}
		else
		{
			// Add this file
			strcpy(fullpath, dirname);
			strcat(fullpath, "\\");
			strcat(fullpath, fd.name);
			monitoredFiles.add(fullpath);
			//Log(" Adding file: %s", fullpath);
		}
		// Find the next file
		if (_findnext(hFile, &fd))
			break;
	}
	if (hFile != -1)
		_findclose(hFile);
}

// Create a list of files to monitor
void createMonitoredFileList(void)
{
	Log("[createMonitoredFileList] Scanning for files in %s", prefBaseMonitorDir);

	monitorDirectory(prefBaseMonitorDir);

	Log("[createMonitoredFileList] %d files found", monitoredFiles.count());
}

// Check to make sure each file in the list of files to monitor can be opened
void checkMonitoredFileExistence(void)
{
	FILE *fp;
	int failures = 0, successes = 0;

	// Make sure there's at least one file to monitor before proceeding
	if (!monitoredFiles.count())
	{
		Log("[checkMonitoredFileExistence] Warning: No files being monitored - check configuration");
		return;
	}

	// Loop through the files to be monitored and attempt to open each one
	char *str;
	for (monitoredFiles.reset(); str = monitoredFiles.next();)
	{
		fp = fopen(str, "rb");
		if (!fp)
		{
			failures++;
			Log("[checkMonitoredFileExistence] Warning: file not found: %s", str);
		}
		else
		{
			successes++;
			fclose(fp);
		}
	}

	// Log the results
	if (successes)
		Log("[checkMonitoredFileExistence] %d files present and accounted for", successes);
	if (failures)
		Log("[checkMonitoredFileExistence] Warning: %d files could not be opened", failures);
}

// Given an int date/time, returns a printable string representation of it
char *intTimeToString(int inttime)
{
	static char str[64];
	struct tm t, *tp;

	tp = localtime((time_t *) &inttime);
	memcpy(&t, tp, sizeof(struct tm));
	strcpy(str, asctime(&t));
	if (strlen(str) > 1)
		str[strlen(str) - 1] = '\0';

	return str;
}

// Returns the last accessed time of the given file, as an int (time_t)
int getLastAccessTime(char *fn)
{
	FILETIME ft, lt;
	SYSTEMTIME st;
	struct tm t;
	time_t inttime;

	// Try to open the file
	HANDLE hFile = CreateFile(fn, GENERIC_READ, 
		FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);

	// If opening the file didn't work...
	if (hFile == INVALID_HANDLE_VALUE)
	{
		// only warn about it once...
		if (!monitoredFiles.getValueInt(fn, "warned"))
		{
			Log("[getLastAccessTime] Warning: Unable to open %s", fn);
			monitoredFiles.setValueInt(fn, "warned", 1);
		}
		return 0;
	}
	// ...until the file becomes openable again.
	monitoredFiles.setValueInt(fn, "warned", 0);

	// Get the last access time and close the file
	GetFileTime(hFile, NULL, &ft, NULL);
	CloseHandle(hFile);

	// Convert the access time from file time to local time, 
	// then system time, then tm, then time_t (whew)
	FileTimeToLocalFileTime(&ft, &lt);
	FileTimeToSystemTime(&lt, &st);
	memset(&t, 0, sizeof(struct tm));
	t.tm_mon = st.wMonth - 1;
	t.tm_mday = st.wDay;
	t.tm_year = st.wYear - 1900;
	t.tm_hour = st.wHour - 1;
	t.tm_min = st.wMinute;
	t.tm_sec = st.wSecond;
	inttime = mktime(&t);

	// Return the access time as an int
	return (int) inttime;
}

// Initializes the last accessed times of the given files
void initAccessTimes(void)
{
	char *str;
	for (monitoredFiles.reset(); str = monitoredFiles.next();)
	{
		monitoredFiles.setValueInt(str, "accessTime", getLastAccessTime(str));
	}
}

// Returns the first directory that a file is in.. for example, 
// given "c:\AtronCmds\AudioTron1\blah.mp3", this function returns "AudioTron1"
char *getDirectory(char *fn)
{
	static char output[MAX_PATH];
	char op[MAX_PATH];

	char *ch = strrchr(fn, '\\');
	if (ch && ch > fn)
	{
		char *ch2 = ch - 1;
		while (ch2 > fn && *ch2 != '\\')
			ch2--;
		if (ch2 != fn)
		{
			strncpy(op, ch2 + 1, ch - ch2 - 1);
			op[ch - ch2 - 1] = '\0';
			strcpy(output, op);
			return output;
		}
		else
			return NULL;
	}
	else
		return NULL;

	return output;
}

// Take action when a file's access time has changed
void logAccess(int accessTime, char *fn)
{
	// The file has been accessed - log a message and send a message to the VB program
	char buf[512];
	sprintf(buf, "%s - ACCESS: %s", intTimeToString(accessTime), getBaseFilename(fn));
	Log("%s", buf);

	// Initiate playlist generation sequence if this is the right file
	if (!stricmp(getBaseFilename(fn), prefPlaylistTriggerFilename))
	{
		// Get the name (or IP) of the AudioTron accessing this file
		char *fndir = getDirectory(fn);
		if (!fndir)
		{
			Log("[logAccess] ERROR: File is not in a valid location: %s", fn);
			return;
		}
		Log("[logAccess] Trigger file accessed in %s", fndir);

		// If playlist generation is already in progress, this means one of two things:
		// 1) The AudioTron that initiated the playlist generation is doing so again
		// 2) A different AudioTron is trying to initiate playlist generation before 
		//    the first has finished.
		if (playlistGenerationInProgress)
		{
			if (!stricmp(playlistGenerationATName, fndir))
			{
				// It's the same AudioTron that initiated the current playlist generation...
				// Don't do anything.
				return;
			}
			else
			{
				// Another AudioTron is trying to initiate playlist generation - tell them to go away
				ATSetMessage(fndir, prefMsgLine1Busy, prefMsgLine2Busy, prefMsgDelayBusy);
			}
		}
		else
		{
			// Playlist generation was NOT in progress, so start it!
			// Remember which AudioTron initiated this playlist generation
			strcpy(playlistGenerationATName, fndir);

			// Switch to playlist generation state and start a timer
			Log("[logAccess] Playlist generation: Initiating playlist generation sequence");
			playlistGenerationInProgress = TRUE;
			playlistGenerationStartTime = timeGetTime();
			// Just in case something else is playing (and the special playlist got queued), 
			// stop playback, clear the playback queue, and wait for a brief moment.
			Log("[logAccess] Playlist generation: Issuing stop command");
			ATCommand(playlistGenerationATName, "stop");
			Log("[logAccess] Playlist generation: Issuing clear command");
			ATCommand(playlistGenerationATName, "clear");
			// Tell the user to choose a song
			Log("[logAccess] Playlist generation: Setting 'choose' message");
			ATSetMessage(playlistGenerationATName, prefMsgLine1ChooseSong, prefMsgLine2ChooseSong, prefMsgDelayChooseSong);
		}
	}

	// NOTE - ignore the following commented out code, it's for inter-process 
	// communication with a VB program, which is not necessary in the current system
	//
	// Send a message to the HVMon VB program via WM_COPYDATA IPC
	// First, find the IPC server window's handle
	/*HWND hwndVB = FindWindow("STATIC", "HVMon IPC - Server");
	if (hwndVB)
	{
		// Send the data via WM_COPYDATA
		COPYDATASTRUCT cds;
		cds.dwData = 42;	// arbitrary value
		cds.lpData = buf;
		cds.cbData = strlen(buf) + 1;
		SendMessage(hwndVB, WM_COPYDATA, NULL, (LPARAM) &cds);
	}*/
}

// Generate a playlist!
void generatePlaylist(char *listType, char *seed)
{
	char fn[MAX_PATH];

	if (!stricmp(listType, "song"))
		strcpy(fn, seed);

	// Prepare the AudioTron for playlist generation
	Log("[generatePlaylist] Playlist generation: Issuing stop command");
	ATCommand(playlistGenerationATName, "stop");
	Log("[generatePlaylist] Playlist generation: Issuing clear command");
	ATCommand(playlistGenerationATName, "clear");
	Log("[generatePlaylist] Playlist generation: Setting 'generation' message");
	ATSetMessage(playlistGenerationATName, "Generating playlist from:", getBaseFilename(fn), 10);

	// Get rid of any result file from a previous playlist generation
	char playlistResultFilename[MAX_PATH];
	strcpy(playlistResultFilename, appPath);
	strcat(playlistResultFilename, "plresult.txt");
	unlink(playlistResultFilename);

	// Write out the config file for MakePlaylist, describing what to make the playlist from
	char playlistConfigFilename[MAX_PATH];
	strcpy(playlistConfigFilename, appPath);
	strcat(playlistConfigFilename, "pl.txt");
	FILE *fp = fopen(playlistConfigFilename, "w");
	if (fp)
	{
		Log("[generatePlaylist] Playlist generation: Filename from AT is %s", fn);
		// Write out the type of playlist we want to generate
		fprintf(fp, "%s\n", listType);
		if (!stricmp(listType, "song"))
		{
			// Write out the seed filename, WITHOUT the UNC server name and share name
			char playlistSeedFilename[MAX_PATH];
			char *ch = strchr(fn + 2, '\\');
			_ASSERTE(ch);
			ch = strchr(ch + 1, '\\');
			_ASSERTE(ch);
			ch++;
			strcpy(playlistSeedFilename, ch);
			Log("[generatePlaylist] Playlist generation: Trying to match filename %s", playlistSeedFilename);
			fprintf(fp, "%s\n", playlistSeedFilename);
		}
		else
		{
			Log("[generatePlaylist] ERROR - unknown playlist type %s", listType);
			return;
		}
		// Write out the playlist output filename and result filename
		char playlistOutputFilename[MAX_PATH];
		if (!strchr(prefPlaylistOutputFilename, '\\'))
			sprintf(playlistOutputFilename, "%s\\%s\\%s", prefBaseMonitorDir, playlistGenerationATName, prefPlaylistOutputFilename);
		else
			strcpy(playlistOutputFilename, prefPlaylistOutputFilename);
		fprintf(fp, "%s\n", playlistOutputFilename);
		fprintf(fp, "%s\n", playlistResultFilename);
		fclose(fp);

		// Generate the playlist! (spawn MakePlaylist.exe to do it)
		char makePlaylistFilename[MAX_PATH];
		strcpy(makePlaylistFilename, appPath);
		strcat(makePlaylistFilename, prefPlaylistGeneratorFilename);
		Log("[generatePlaylist] Playlist generation: Asking %s to write %s", makePlaylistFilename, playlistOutputFilename);
		_spawnl(_P_WAIT, makePlaylistFilename, makePlaylistFilename, playlistConfigFilename, NULL);

		// See if the playlist generation was successful by looking at the result file
		fp = fopen(playlistResultFilename, "r");
		if (fp)
		{
			char playlistResult[256];
			fgets(playlistResult, sizeof(playlistResult) - 1, fp);
			if (playlistResult[0])
				playlistResult[strlen(playlistResult) - 1] = '\0';
			if (!stricmp(playlistResult, "success"))
			{
				// Playlist generation was successful!  Queue the playlist and start playback.
				Log("[generatePlaylist] Playlist generation: Successfully created playlist!");
				Log("[generatePlaylist] Playlist generation: Queuing the playlist");
				ATQueueFile(playlistGenerationATName, "list", getBaseFilename(prefPlaylistOutputFilename));
				Log("[generatePlaylist] Playlist generation: Issuing play command");
				ATSetMessage(playlistGenerationATName, prefMsgLine1Successful, prefMsgLine2Successful, prefMsgDelaySuccessful);
				ATCommand(playlistGenerationATName, "play");
				playlistGenerationInProgress = FALSE;
			}
			else
			{
				// Display a message indicating that the song was not mixable or not found in the DB
				ATSetMessage(playlistGenerationATName, prefMsgLine1Failed, prefMsgLine2Failed, prefMsgDelayFailed);
				Log("[generatePlaylist] Playlist generation: Song was not mixable or not found in the DB");
			}
			fclose(fp);
		}
		else
		{
			Log("[generatePlaylist] Error: Unable to open result file '%s'", playlistResultFilename);
			ATSetMessage(playlistGenerationATName, "Playlist generation failed:", "Unable to open result file.", 5);
			playlistGenerationInProgress = FALSE;
		}
	}
	else
	{
		Log("[generatePlaylist] Error: Unable to write config file '%s'", playlistConfigFilename);
		ATSetMessage(playlistGenerationATName, "Playlist generation failed:", "Unable to write config file.", 5);
		playlistGenerationInProgress = FALSE;
	}
}

// Handle an ongoing playlist generation session - this function is called on a regular basis
void handlePlaylistUI(void)
{
	int now = timeGetTime();

	// Stop the playlist generation session if the user hasn't done anything in awhile
	if (playlistGenerationInProgress && now - playlistGenerationStartTime > prefUserTimeout)
	{
		// Stop the playlist generation session
		playlistGenerationInProgress = FALSE;

		// Display a message to let the user know that they timed out and will have to start over
		ATSetMessage(playlistGenerationATName, prefMsgLine1Timeout, prefMsgLine2Timeout, prefMsgDelayTimeout);
		return;
	}

	// Don't start checking for the song until a certain amount of time has passed...
	if (playlistGenerationInProgress && now - playlistGenerationStartTime > prefDelayBeforeSongCheck)
	{
		// Poll every two seconds
		if (now - playlistGenerationLastPollTime > prefSongCheckInterval)
		{
			Log("[handlePlaylistUI] Playlist generation: Checking to see if user chose a song yet");
			// Get the AudioTron's status
			char status[4096];
			if (!ATGetStatus(playlistGenerationATName, status, sizeof(status)))
			{
				// Look in the results to see if something is playing
				char *ch = strstr(status, "SourceID=");
				if (ch)
				{
					// Get the filename of what's playing
					ch += strlen("SourceID=");
					char fn[MAX_PATH];
					char *chfn = fn;
					while (*ch && *ch != '\r' && *ch != '\n')
						*chfn++ = *ch++;
					*chfn = '\0';
					Log("[handlePlaylistUI] Playlist generation: File is playing: %s", getBaseFilename(fn));
					// Ignore the dummy file used in the trigger playlist
					if (stricmp(getBaseFilename(fn), prefPlaylistDummyFilename))
					{
						// Generate the playlist!
						generatePlaylist("song", fn);
					}
				}
			}
			playlistGenerationLastPollTime = timeGetTime();
		}
	}
}

// Monitor the list of files for changes in access time
void monitorFiles(void)
{
	Log("[monitorFiles] Entering monitoring loop");

	// This is the main monitoring loop - sleeping is done in this call to WaitForSingleObject()
	while (WaitForSingleObject(quitMonitor, prefAccessTimePollInterval) != WAIT_OBJECT_0)
	{
		// Loop through the monitored files
		char *str;
		for (monitoredFiles.reset(); str = monitoredFiles.next();)
		{
			// Get the last access time of this file
			int accessTime = getLastAccessTime(str);
			if (accessTime)
			{
				// Compare it to the previously recorded last access time
				int oldAccessTime = monitoredFiles.getValueInt(str, "accessTime");
				if (accessTime != oldAccessTime)
				{
					// Log the access
					logAccess(accessTime, str);

					// Save the new access time
					monitoredFiles.setValueInt(str, "accessTime", accessTime);
				}
			}
		}

		// We're also going to do polling here, every now and then, if playlist 
		// generation is in progress, to find out when the user starts playing a song.
		handlePlaylistUI();
	}
}

// Thread to monitor file access times (this is the real heart of the program)
unsigned int __stdcall monitor_thread(void *ptr)
{
	// Create an event which will signal termination of the monitoring thread
	quitMonitor = CreateEvent(NULL, TRUE, FALSE, NULL);
	
	// Create and initialize the list of files to monitor
	createMonitoredFileList();
	checkMonitoredFileExistence();
	initAccessTimes();

	// Main monitoring loop - monitor for changes in last access time
	monitorFiles();
	
	// Get rid of the monitoring thread termination event
	CloseHandle(quitMonitor);

	return 0;
}

// Set a string preference (returns 0 on success, 1 on failure)
int setPrefString(char *keyname, char *key, char *value, char *pref)
{
	if (!stricmp(keyname, key))
	{
		strcpy(pref, value);
		return 0;
	}
	else
		return 1;
}

// Set an int preference (returns 0 on success, 1 on failure)
int setPrefInt(char *keyname, char *key, char *value, int *pref)
{
	if (!stricmp(keyname, key))
	{
		*pref = atoi(value);
		return 0;
	}
	else
		return 1;
}

// Reads in a config file, setting preferences
int readConfigFile(void)
{
	// Open the config file
	char fn[MAX_PATH];
	strcpy(fn, appPath);
	strcat(fn, "acwatch.ini");
	FILE *fp = fopen(fn, "r");
	if (!fp)
	{
		Log("[readConfigFile] WARNING: Unable to open config file, using defaults (was looking for %s)", fn);
		return 1;
	}

	// Read each line of the config file
	char buf[1024], key[1024], value[1024];
	char *ch;
	while (fgets(buf, sizeof(buf), fp))
	{
		// Get rid of the newline at the end of the line
		if (buf[0])
			buf[strlen(buf) - 1] = '\0';

		// Ignore blank lines and comments
		if (!buf[0] || buf[0] == ' ' || buf[0] == ';' || buf[0] == '/' || buf[0] == '\'' || buf[0] == '#')
			continue;

		// Get the name of the setting
		ch = strchr(buf, '=');
		if (!ch)
			continue;
		strncpy(key, buf, ch - buf);
		key[ch - buf] = '\0';
		
		// Get the value of the setting, being flexible about quoted strings
		ch++;
		if (!*ch)
			continue;
		if (*ch == '\"')
		{
			ch++;
			if (!*ch)
				continue;
			strcpy(value, ch);
			value[strlen(value) - 1] = '\0';
		}
		else
			strcpy(value, ch);

		// See if we can match this setting up with a global variable
		if (!setPrefInt("AccessTimePollInterval", key, value, &prefAccessTimePollInterval))
			continue;
		if (!setPrefInt("DelayBeforeSongCheck", key, value, &prefDelayBeforeSongCheck))
			continue;
		if (!setPrefInt("SongCheckInterval", key, value, &prefSongCheckInterval))
			continue;
		if (!setPrefInt("UserTimeout", key, value, &prefUserTimeout))
			continue;
		if (!setPrefString("BaseMonitorDir", key, value, prefBaseMonitorDir))
			continue;
		if (!setPrefString("PlaylistTriggerFilename", key, value, prefPlaylistTriggerFilename))
			continue;
		if (!setPrefString("PlaylistDummyFilename", key, value, prefPlaylistDummyFilename))
			continue;
		if (!setPrefString("PlaylistOutputFilename", key, value, prefPlaylistOutputFilename))
			continue;
		if (!setPrefString("PlaylistGeneratorFilename", key, value, prefPlaylistGeneratorFilename))
			continue;
		if (!setPrefString("ATPassword", key, value, prefATPassword))
			continue;
		if (!setPrefString("MsgLine1ChooseSong", key, value, prefMsgLine1ChooseSong))
			continue;
		if (!setPrefString("MsgLine2ChooseSong", key, value, prefMsgLine2ChooseSong))
			continue;
		if (!setPrefInt("MsgDelayChooseSong", key, value, &prefMsgDelayChooseSong))
			continue;
		if (!setPrefString("MsgLine1Successful", key, value, prefMsgLine1Successful))
			continue;
		if (!setPrefString("MsgLine2Successful", key, value, prefMsgLine2Successful))
			continue;
		if (!setPrefInt("MsgDelaySuccessful", key, value, &prefMsgDelaySuccessful))
			continue;
		if (!setPrefString("MsgLine1Failed", key, value, prefMsgLine1Failed))
			continue;
		if (!setPrefString("MsgLine2Failed", key, value, prefMsgLine2Failed))
			continue;
		if (!setPrefInt("MsgDelayFailed", key, value, &prefMsgDelayFailed))
			continue;
		if (!setPrefString("MsgLine1Busy", key, value, prefMsgLine1Busy))
			continue;
		if (!setPrefString("MsgLine2Busy", key, value, prefMsgLine2Busy))
			continue;
		if (!setPrefInt("MsgDelayBusy", key, value, &prefMsgDelayBusy))
			continue;
		if (!setPrefString("MsgLine1Timeout", key, value, prefMsgLine1Timeout))
			continue;
		if (!setPrefString("MsgLine2Timeout", key, value, prefMsgLine2Timeout))
			continue;
		if (!setPrefInt("MsgDelayTimeout", key, value, &prefMsgDelayTimeout))
			continue;
	}

	// Close the config file
	fclose(fp);

	Log("[readConfigFile] Processed config file %s", fn);

	return 0;
}