// tgRegistry class for accessing the Windows registry
//
// (c) 2001 by Tom Grandgent, started on 12/6/2001

#include "tgRegistry.h"

void tgRegistry::initDefaultValues(void)
{
	hive = HKEY_CURRENT_USER;
	key = NULL;
	keyName = NULL;
	samDesired = KEY_ALL_ACCESS;
}

tgRegistry::tgRegistry(void)
{
	initDefaultValues();
}

tgRegistry::tgRegistry(char *newKeyName)
{
	initDefaultValues();
	useKey(newKeyName);
}

tgRegistry::tgRegistry(HKEY newHive, char *newKeyName)
{
	initDefaultValues();
	useHive(newHive);
	useKey(newKeyName);
}

tgRegistry::~tgRegistry(void)
{
	closeKey();
}

void tgRegistry::useHive(HKEY newHive)
{
	hive = newHive;
}

void tgRegistry::useKey(char *newKeyName)
{
	if (keyName)
	{
		free(keyName);
		keyName = NULL;
	}
	keyName = strdup(newKeyName);
}

void tgRegistry::closeKey(void)
{
	if (key)
		RegCloseKey(key);
	key = NULL;
}

LONG tgRegistry::createKey(void)
{
	LONG err;

	closeKey();
	err = RegCreateKeyEx(hive, keyName, 0, "", 
		REG_OPTION_NON_VOLATILE, samDesired, NULL, &key, (unsigned long *) &disposition);

	return err;
}

LONG tgRegistry::createKey(char *newKeyName)
{
	useKey(newKeyName);
	return createKey();
}

LONG tgRegistry::openKey(void)
{
	closeKey();
	return RegOpenKeyEx(hive, keyName, 0, samDesired, &key);
}

LONG tgRegistry::openKey(char *newKeyName)
{
	OutputDebugString("tgRegistry::openKey()\n");
	useKey(newKeyName);
	return openKey();
}

LONG tgRegistry::setValue(char *name, DWORD type, void *data, DWORD dataLen)
{
	if (!key)
		createKey();
	return RegSetValueEx(key, name, 0, type, (CONST BYTE *) data, dataLen);
}

LONG tgRegistry::setValueInt(char *name, DWORD data)
{
	return setValue(name, REG_DWORD, &data, sizeof(DWORD));
}

LONG tgRegistry::setFlag(char *name, int flagTrue)
{
	return setValueInt(name, flagTrue);
}

LONG tgRegistry::setValueString(char *name, char *data)
{
	return setValue(name, REG_SZ, data, strlen(data));
}

LONG tgRegistry::getValue(char *name, void *data, DWORD dataLen)
{
	DWORD cbData = dataLen;

	if (!key)
		openKey();

	return RegQueryValueEx(key, name, 0, NULL, (BYTE *) data, &cbData);
}

LONG tgRegistry::getValueInt(char *name, int *data)
{
	return getValue(name, data, sizeof(DWORD));
}

LONG tgRegistry::getValueInt(char *name, int *data, int defaultValue)
{
	LONG ret = getValue(name, data, sizeof(DWORD));
	if (ret != ERROR_SUCCESS)
		*data = defaultValue;

	return ret;
}

int tgRegistry::getFlag(char *name)
{
	int data;

	LONG ret = getValue(name, &data, sizeof(DWORD));
	if (ret == ERROR_SUCCESS && data)
		return TRUE;
	
	return FALSE;
}

LONG tgRegistry::getValueString(char *name, char *data, DWORD dataLen)
{
	return getValue(name, data, dataLen);
}

LONG tgRegistry::getValueString(char *name, char *data, DWORD dataLen, char *defaultValue)
{
	LONG ret = getValue(name, data, dataLen);
	if (ret != ERROR_SUCCESS)
		strcpy(data, defaultValue);

	return ret;
}
