// tgStrList by Tom Grandgent
//
// Class for managing a list of strings and associated information (key/value pairs) with each string
// (this is basically a dictionary now)
//
// Note: *Removing* strings, keys, or values is not implemented.  Only add/set/get.

#ifndef TGSTRLIST_INCLUDED
#define TGSTRLIST_INCLUDED

#include <stdlib.h>
#include <string.h>

// Linked list of strings
typedef struct strlist_t
{
	char *str;
	void *data;
	struct strlist_t *subList;
	struct strlist_t *subListTail;
	struct strlist_t *next;
} strlist_t;

class tgStrList
{
private:
	// List pointers and # of items in the list
	strlist_t *root, *tail, *nextItem;
	int itemCount;

	// Returns the linked list node for the given string
	strlist_t *findString(char *str);

	// Adds the given string to the given linked list
	strlist_t *addNode(strlist_t **rootNode, strlist_t **tailNode, char *newStr);

public:
	// Constructor
	tgStrList();

	// Add a string to the list
	void add(char *newStr);

	// Reset enumeration
	void reset(void);

	// Return next string in enumeration
	char *next(void);

	// Gets data for a particular string
	void *getValue(char *str, char *key);

	// Gets an int for a particular string
	int getValueInt(char *str, char *key);

	// Gets a string for a particular string
	char *getValueString(char *str, char *key);

	// Sets data for a particular string - returns 0 on success, 1 on error
	int setValue(char *str, char *key, void *val);

	// Sets an int for a particular string - returns 0 on success, 1 on error
	int setValueInt(char *str, char *key, int val);

	// Sets a string for a particular string - returns 0 on success, 1 on error
	int setValueString(char *str, char *key, char *val);

	// Returns the number of items in the list
	int count(void) { return itemCount; }
};

#endif